/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
 
 
var map, selectControl, selectedFeature, previousSelectedFeature, geolocationPoint, geolocationAccuracy, selectedMarker, locationAvailable, mapCenter;
var db;
var currentView, myLocation;
var MAGIC_DATE;
var dbRequestKey = "";
var storage = window.localStorage;
var selectFeatureTrigger = false;

var selectedCollection;
var invisibleStyle, defaultParkingArea, defaultParkingAreaSelected, defaultParkingIcon, defaultParkingSelectedIcon, defaultParkingGarageIcon, defaultParkingGarageSelectedIcon, defaultParkingTerrainIcon, defaultParkingTerrainSelectedIcon, defaultParkRideIcon, defaultParkRideSelectedIcon, myLocationStyle, selectedMarkerStyle;
var myLocationWatchID;
var blocks = new Array();
var blocksLoaded = new Array();
var listenerKeys = new Array();

var debugTime = new Date().getTime();
var geocoder = new google.maps.Geocoder();

var app = {	
    // Application Constructor
    initialize: function() {
		//make sure the map covers the whole canvas
		document.getElementById('geolocation').parentNode.setAttribute('style',document.getElementById('geolocation').parentNode.getAttribute('style') + '; height: 100%');
		//initialize data
		setBlocks();
		setDefaultStyles();
		//set MAGIC_DATE
		var dateObj = new Date();
		var yyyy = dateObj.getFullYear();
		var mm = dateObj.getMonth()+1;
		if(mm<10){mm='0'+mm}
		var dd = dateObj.getDate();
		if(dd<10){dd='0'+dd}
		MAGIC_DATE = yyyy + "" + mm + "" + dd;
		this.bindEvents();
    },
    // Bind Event Listeners
    //
    // Bind any events that are required on startup. Common events are:
    // 'load', 'deviceready', 'offline', and 'online'.
    bindEvents: function() {
        document.addEventListener('deviceready', this.onDeviceReady, false);
    },	
	
    // deviceready Event Handler
    //
    // The scope of 'this' is the event. In order to call the 'receivedEvent'
    // function, we must explicitly call 'app.receivedEvent(...);'
    onDeviceReady: function() {
		var dbPath = "/data/data/com.dennisstam.parkeerApp/app_database/spatial.db";
		if(typeof db === 'undefined') {
			//console.log("onDeviceReady: beforeDBopen: " + (new Date().getTime()-debugTime));
			db = window.sqlitePlugin.openDatabase({name: dbPath});
		}
		//console.log("onDeviceReady: afterDBopen: " + (new Date().getTime()-debugTime));
		if (navigator.geolocation) {
			locationAvailable = true;
			myLocationWatchID = navigator.geolocation.watchPosition(onGeoLocationReceive, onGeoLocationReceiveError, {  enableHighAccuracy: true,  timeout: 10000,  maximumAge: 60000 });
			//console.log("onDeviceReady: afterPosition: " + (new Date().getTime()-debugTime));
			app.onSuccess();
		} else {
			locationAvailable = false;
			app.onSuccess();
		}
    },
	
	onError: function() {
			locationAvailable = false;
			app.onSuccess();
	},
	
    onSuccess: function(position) {
		jQuery(function() {      
			//Enable swiping...
			jQuery("#locationHeader").swipe( {
				//Generic swipe handler for all directions
				swipe:function(event, direction, distance, duration, fingerCount, fingerData) {
					switch(direction) {
						case "up":
							jQuery(":animated").promise().done(function() {
								if(!(regulationsVisible())) {
									showRegulations();
									hideMapControls();
								}
							});						
							break;
						case "down":
							jQuery(":animated").promise().done(function() {
								if(regulationsVisible()) {
									hideRegulations();
									showMapControls();
									return;
									//break;
								}
								//hide last block
								var tmpUndefined;
								onFeatureSelect(tmpUndefined, null);
							});
							break;
						default:
							break;
					}
				},
				//And click...
				tap:function(event, target) {
						switch(regulationsVisible()) {
							case true:
								jQuery(":animated").promise().done(function() {
									hideRegulations();
									showMapControls();
								});
								break;
							case false:
								jQuery(":animated").promise().done(function() {
									showRegulations();
									hideMapControls();
								});						
								break;
							default:
								break;
						}
				},
				longTap:function(event, target) {
					//TODO add navigation link using demos.jquerymobile.com/1.3.0-rc.1/docs/demos/widgets/popup/
					alert("longtap!");
				},
				//Default is 75px, set to 0 for demo so any distance triggers swipe
				threshold:10
			});
		});
		
		if(locationAvailable && !(typeof selectedFeature === 'undefined') && !(typeof position === 'undefined')) {
			myLocation = new google.maps.LatLng(position.coords.latitude, position.coords.longitude);
		} else {
			if(storage.getItem("firstRun") == 'false') {
				if(!(isNaN(storage.getItem("lastLocationLng")) || isNaN(storage.getItem("lastLocationLat"))))
				{
					myLocation = new google.maps.LatLng(storage.getItem("lastLocationLat"),storage.getItem("lastLocationLng"));	
				} else {
					//something went wrong with the coordinates last time... lets fix
					if(!(typeof position === 'undefined'))
					{
						myLocation = new google.maps.LatLng(position.coords.latitude, position.coords.longitude);
					} else {
						myLocation = new google.maps.LatLng(51.924327478661525,4.479288228954452);
					}
				}
			} else {
				storage.setItem("firstRun","false")
				myLocation = new google.maps.LatLng(51.924327478661525,4.479288228954452);	
			}
		}
		
		
		view = new ol.View2D();
		view.on('change:center', function() {
		  var center = ol.proj.transform(view.getCenter(), 'EPSG:3857', 'EPSG:4326');
//		  gmap.setCenter(new google.maps.LatLng(center[1], center[0]));
		  loadAdditionalBlocks();
		  storage.setItem("lastLocationLat", center[1]);
		  storage.setItem("lastLocationLng", center[0]);
		});
		view.on('change:resolution', function() {
//			gmap.setZoom(view.getZoom());
			if(view.getResolution() < 77) {
				mapZoomWarning.style.display = 'none';
				//selectedItemContainer.style.display = 'block';
			} else {
				mapZoomWarning.style.display = 'block';
				//selectedItemContainer.style.display = 'none';
			}
		});

		//console.log("main: beforeAreas: " + (new Date().getTime()-debugTime));
		var vectorAreas = new ol.layer.Vector({
			source: new ol.source.KML({
			  projection: 'EPSG:3857',
			  url: 'kml/empty.kml'
			}),
			maxResolution: 77
		})

		//console.log("main: beforePoints: " + (new Date().getTime()-debugTime));
		var vectorPoints = new ol.layer.Vector({
			source: new ol.source.KML({
				projection: 'EPSG:3857',
				url: 'kml/empty.kml'
			}),
			maxResolution: 77
		});

		//My Location marker
		geolocationPoint = new ol.Feature({
		  geometry: new ol.geom.Point([0,0]),
		  name: 'MyLocation',
		});
		geolocationPoint.setStyle(invisibleStyle);
		geolocationAccuracy = new ol.Feature({
		  geometry: new ol.geom.Circle([0,0],0),
		  name: 'MyAccuracy',
		});
		geolocationAccuracy.setStyle(invisibleStyle);
		var vectorMyLocationSource = new ol.source.Vector({
		  features: [geolocationPoint,geolocationAccuracy]
		})
		var vectorMyLocation = new ol.layer.Vector({
		  source: vectorMyLocationSource
		});

		selectedMarker = new ol.Feature({
		  geometry: new ol.geom.Point([0,0]),
		  name: 'selectedMarker',
		});
		selectedMarker.setStyle(invisibleStyle);
		var vectorSelectedMarkerSource = new ol.source.Vector({
		  features: [selectedMarker]
		})
		var vectorSelectedMarker = new ol.layer.Vector({
		  source: vectorSelectedMarkerSource
		});
		

//custom MyLocation control
		app.MoveToMyLocationControl = function(opt_options) {
		  var options = opt_options || {};
		
		  var anchor = document.createElement('a');
		  anchor.href = '#myLocation';
		  anchor.innerHTML = '<div id="gmapMyLocation" style="z-index: 1000000; position: absolute; right: .5em; top: 1.5em;"><a target="_blank" href="" style="position: static; overflow: visible; float: none; display: inline;"><div style="width: 3em; height: 3em; cursor: pointer;"><img src="img/mylocation.png" draggable="false" style="position: absolute; left: 0px; top: 0px; -webkit-user-select: none; border: 0px; padding: 0px; margin: 0px; width:100%; height:100%;"></div></a></div>';

		  var this_ = this;
		  var moveToMyLocation = function(e) {
			// prevent #rotate-north anchor from getting appended to the url
			e.preventDefault();
			this_.getMap().getView().setCenter(ol.proj.transform([myLocation.lng(), myLocation.lat()], 'EPSG:4326', 'EPSG:3857'));
		  };
		
		  anchor.addEventListener('click', moveToMyLocation, false);
		  anchor.addEventListener('touchstart', moveToMyLocation, false);
		
		  var element = document.createElement('div');
		  element.className = 'myLocation ol-unselectable';
		  element.appendChild(anchor);
		
		  ol.control.Control.call(this, {
			element: element,
			target: options.target
		  });
		
		};
		ol.inherits(app.MoveToMyLocationControl, ol.control.Control);

//set submit on 'enter' on search-field
		jQuery('#searchAddress').keypress(function(e){
			if(e.keyCode==13)
			jQuery('#searchAddressButton').click();
		});


		//console.log("main: beforeRender: " + (new Date().getTime()-debugTime));
		//Render map
		var olMapDiv = document.getElementById('olmap');
		map = new ol.Map({
		  controls: ol.control.defaults().extend([
			new app.MoveToMyLocationControl()
		  ]),
		  layers: [
				new ol.layer.Tile({
					source: new ol.source.MapQuest({layer: 'osm'})
				}),
				vectorAreas, 
				vectorPoints, 
				vectorMyLocation, 
				vectorSelectedMarker
			],
		  interactions: ol.interaction.defaults({
			altShiftDragRotate: false,
			pinchRotate:false,
			pan: false,
			rotate: false
		  }).extend([new ol.interaction.DragPan({kinetic: false})]),
		  target: 'gmap',
		  ol3Logo: false,
		  view: view
		});

		map.once('postrender', function(){
			// do something only the first time the map is loaded
			geolocationLoading.style.display = 'none';
			searchAddressContainer.style.display = 'block';
		});
		
		document.getElementById('gmap').addEventListener('click', function(evt) {
			mapClickAtLocation([evt.layerX,evt.layerY])
		}, false);
		
		view.setCenter(ol.proj.transform([myLocation.lng(), myLocation.lat()], 'EPSG:4326', 'EPSG:3857'));
		view.setZoom(14);
		
		olMapDiv.parentNode.removeChild(olMapDiv);
//		gmap.controls[google.maps.ControlPosition.TOP_LEFT].push(olMapDiv);
		selectFeatureTrigger = true;		
//console.log("main: DONE: " + (new Date().getTime()-debugTime));
	},
};

function mapClickAtLocation(clickedPosition, mapcoordinates) {
	if(!(typeof mapcoordinates === 'undefined')) {
		clickedPosition = map.getPixelFromCoordinate(mapcoordinates);
	}
	
	storage.setItem("lastClickedPosition", JSON.stringify(clickedPosition));
	//TODO: store ID of feature as well to select POINTs as well
	
	var centerPoint = map.getCoordinateFromPixel(clickedPosition);
	var clickedFeatsAreas  = map.getLayers().getAt(1).getSource().getFeaturesAtCoordinate(centerPoint);
	
	var mapResolution = view.getResolution();
	var mapDimensions = map.getSize();
	
	var hFactor = 20;
	var vFactor = 15
	
	var myExt = ol.extent.boundingExtent([[(centerPoint[0] - (mapResolution*mapDimensions[0])/hFactor),(centerPoint[1] - (mapResolution*mapDimensions[1])/vFactor)],
		[(centerPoint[0] + (mapResolution*mapDimensions[0])/hFactor),(centerPoint[1] + (mapResolution*mapDimensions[1])/vFactor)]]);
	var clickedFeatsPoints = map.getLayers().getAt(2).getSource().getFeaturesInExtent(myExt);
	
	var clickedFeats = clickedFeatsAreas.concat(clickedFeatsPoints);
	if(clickedFeats.length == 1) { 
		moveSelectedMarker(clickedFeats[0], centerPoint);
		onFeatureSelect(clickedFeats[0], centerPoint);	
	} else {
		clickedFeats.sort(function(a, b){
			var nameA=a.get('areaId').toLowerCase(), nameB=b.get('areaId').toLowerCase();
			if (nameA < nameB) //sort string ascending
				return -1 ;
			if (nameA > nameB)
				return 1;
			return 0; //default return value (no sorting)
		})
		var i = findNextFeatAfterSelected(clickedFeats);
		moveSelectedMarker(clickedFeats[i], centerPoint);
		onFeatureSelect(clickedFeats[i], centerPoint);
	}
	return true;
}

function findNextFeatAfterSelected(clickedFeats, clickLocation) {
	if(typeof selectedFeature === 'undefined') { return 0; }
	for (var i=0, iLen=clickedFeats.length; i<iLen; i++) {
		if (clickedFeats[i].get('areaId').toLowerCase() == selectedFeature.get('areaId').toLowerCase()) {
			if(i<(iLen-1)) {				
				return (i+1);	
			} else {
				return 0;
			}
		}
	}
	return 0;
}

function moveSelectedMarker(feature, centerPoint) {
	if(typeof feature === 'undefined')
	{
		//show selectedmarker
		selectedMarker.getGeometry().setCoordinates([centerPoint[0], centerPoint[1]]);
		selectedMarker.setStyle(selectedMarkerStyle);
	} else {
		switch(feature.getGeometry().getType()) {
			case "Point":
				//dont show selectedmarker
				selectedMarker.getGeometry().setCoordinates([centerPoint[0], centerPoint[1]]);
				selectedMarker.setStyle(invisibleStyle);
				break;
			default:
				//show selectedmarker
				selectedMarker.getGeometry().setCoordinates([centerPoint[0], centerPoint[1]]);
				selectedMarker.setStyle(selectedMarkerStyle);
				break;
		}
	}
}

function onFeatureSelect(feature, clickedLatLng) {

	previousSelectedFeature = selectedFeature;
	if(!(typeof previousSelectedFeature === 'undefined'))
	{
		applyVectorStyling(previousSelectedFeature, false)
	}
	
	selectedFeature = feature;
	if(typeof selectedFeature === 'undefined') {
		//nothing selected: hide all
		//check state of slider
		if(jQuery('#locationHeaderText').filter(':visible').length == 1) {
			jQuery('#selectedItemContainer').slideToggle({
			  direction: "up"
			}, 300);
		}
		hideRegulations();
		showMapControls();
		return;
	}
	
	if(feature.getGeometry().getType() == "POINT") {
		//replace clicked coords with actual coords of point
		clickedLatLng = [feature.getGeometry().flatCoordinates[0], feature.getGeometry().flatCoordinates[1]];
	}
	
	//Show the spinner instead of current txt
	jQuery("#locationHeaderTextTxt").hide();
	jQuery("#locationHeaderLoading").show();
	hideRegulations();
	showMapControls();
	jQuery("#selectedItemContainer").removeClass("selectedItemContainerCollapsed");
	jQuery("#selectedItemContainer").removeClass("selectedItemContainerFull");

	//highlight selected feature
	applyVectorStyling(feature, true)
	//set fields
	jQuery("#locationHeaderTextName").text(feature.get('areaIdDesc') + " - " + feature.get('areaManagerIdDesc'));
	var logoSrc = "";
		switch(feature.getGeometry().getType()) {
			case "Point":
				var usageId = feature.get('usageId');
				switch(usageId) {
					case "BETAALDP":
						logoSrc = "icon_parking.png";
						break;
					case "GARAGEP":
						logoSrc = "icon_parkgarage.png";
						break;
					case "TERREINP":
						logoSrc = "icon_parking.png";
						break;
					case "PARKRIDE":
						logoSrc = "icon_p_and_r.png";
						break;
					default:
						logoSrc = "icon_parking.png";
						break;
				}
				break;
			case "Polygon":
				logoSrc = "icon_area.png";
				break;
			default:
				logoSrc = "icon_area.png";
				break;
		}
	jQuery("#locationHeaderImgImg").attr("src","img/" + logoSrc);
	var gglLatLng = ol.proj.transform([clickedLatLng[0],clickedLatLng[1]], 'EPSG:3857', 'EPSG:4326');
	findAddressByPosition(new google.maps.LatLng(gglLatLng[1],gglLatLng[0]));
	//continue after address lookup through showAddress(address)
	
	//(pre)fetch regulations
	dbRequestKey = feature.get('areaId') + "|" + feature.get('areaManagerId') + "|" + feature.get('regulationIds');
	fetchRegulationsForArea(feature.get('areaId'), feature.get('areaManagerId'), feature.get('regulationIds'));
}
function showAddress(address) {
	//callback function
	
	jQuery("#locationHeaderTextAddress").text(address);	
	
	//Show the txt, hide the spinner
	jQuery("#locationHeaderLoading").hide();
	jQuery("#locationHeaderTextTxt").show();
	
	//check state of slider
	if(jQuery('#locationHeaderText').filter(':visible').length == 0) {
		jQuery('#selectedItemContainer').slideToggle({
		  direction: "up"
		}, 300);
	}		
}


function showNewSelectedFeature (feature) { 
	featureInfo.removeEventListener('webkitTransitionEnd' );
	previousSelectedFeature = selectedFeature;
	if(!(typeof previousSelectedFeature === 'undefined'))
	{
		applyVectorStyling(previousSelectedFeature, false)
	}
	
	selectedFeature = feature;
	if(!(typeof selectedFeature === 'undefined'))
	{
		applyVectorStyling(selectedFeature, true);
		/*
				selectedFeatureInfo.innerHTML = '' +
					'<div class="card" style="margin-bottom: 0px;">' +
						'<div class="item item-avatar">' + 
							'<img src="img/logo.png">' + 
							'<h1 style="margin-top: 0px;margin-bottom: 0px;">' + feature.get('areaManagerIdDesc') + '</h1>' + 
							'<span style="color: darkgray;">' + feature.get('areaIdDesc') + '</span>' + 
						'</div>' + 
					'</div>';
		*/
		dbRequestKey = feature.get('areaId') + "|" + feature.get('areaManagerId') + "|" + feature.get('regulationIds');
		fetchRegulationsForArea(feature.get('areaId'), feature.get('areaManagerId'), feature.get('regulationIds'));
		showSelectedFeature();
	}
}



function onFeatureUnselect(feature) {
	map.removePopup(feature.popup);
	feature.popup.destroy();
	feature.popup = null;
}    	

function applyVectorStyling(feature, selected)
{
	switch(feature.getGeometry().getType()) {
		case "Point":
			var usageId = feature.get('usageId');
			switch(usageId) {
				case "BETAALDP":
					if(selected) { feature.setStyle(defaultParkingSelectedIcon); } else { feature.setStyle(defaultParkingIcon); }
					break;
				case "GARAGEP":
					if(selected) { feature.setStyle(defaultParkingGarageSelectedIcon); } else { feature.setStyle(defaultParkingGarageIcon); }
					break;
				case "TERREINP":
					if(selected) { feature.setStyle(defaultParkingTerrainSelectedIcon); } else { feature.setStyle(defaultParkingTerrainIcon); }
					break;
				case "PARKRIDE":
					if(selected) { feature.setStyle(defaultParkRideSelectedIcon); } else { feature.setStyle(defaultParkRideIcon); }
					break;
				default:
					break;
			}
			break;
		case "Polygon":
			if(selected) { 
				feature.setStyle(defaultParkingAreaSelected); 				
			} else { 
				feature.setStyle(defaultParkingArea); 
			}
			break;
		default:
			break;
	}
}

function fetchRegulationsForArea(areaId, areaManagerId, regulationIds) {
	regulationIdsArray = regulationIds.split('|');
	var tmpRegulations = regulationIdsArray.join("', '");
	
	var query = "SELECT '" + (areaId + "|" + areaManagerId + "|" + regulationIds) + "' as dbKey, '' as AreaManagerId, '' as RegulationId, '' as RegulationDesc, 'ZZZ' as RegulationType " + 
				"UNION " + 
				"SELECT distinct '" + (areaId + "|" + areaManagerId + "|" + regulationIds) + "' as dbKey, AreaManagerId, RegulationId, RegulationDesc, RegulationType " + 
				"FROM parkingfare " + 
				"WHERE 1 "+
				" AND AreaManagerId='" + areaManagerId + "' AND RegulationId IN ('"+ tmpRegulations + "') " + 
				"	AND EndDateRegulation > '" + MAGIC_DATE + "' AND (EndDateFare > '" + MAGIC_DATE + "' OR EndDateFare=0 OR EndDateFare='') AND (EndDateFarePart > '" + MAGIC_DATE + "' OR EndDateFarePart=0 OR EndDateFarePart='')  " + 
				"	AND (EndDateTimeFrame > '" + MAGIC_DATE + "000000' OR EndDateTimeFrame='') AND (DateSpecialDay > '" + MAGIC_DATE + "' OR DateSpecialDay='') " + 
				"ORDER BY " + 
				"	RegulationType desc, " +
				"	RegulationId asc;";

/*	
	jQuery("#selectedItemRegulationsCard").html("<div id=\"selectedItemRegulationsCard\" class=\"list card\" style=\"margin: 0; \">" + 
          "<a id=\"selectedItemRegulationsCardLoading\" href=\"#\" class=\"item item-icon-right\">" + 
            "<i class=\"icon ion-loading-a\"></i> Regelingen laden..." + 
          "</a>" + 
        "</div>");
*/
	jQuery("#selectedItemRegulationsCard").html("<a id=\"selectedItemRegulationsCardLoading\" href=\"#\" class=\"item item-icon-right\">" + 
            "<i class=\"icon ion-loading-a\"></i> Regelingen laden..." + 
          "</a>");
	
	//fetch BASIC fares
	db.transaction(function(tx){
		tx.executeSql(
			query, [], function(tx, results){
				var len = results.rows.length.valueOf();
				if(len>1) {
					if(dbRequestKey == results.rows.item(0).dbKey)
					{
						for (var i=1; i<len; i++){
							var regulationId = results.rows.item(i).RegulationId;
							var regulationType = results.rows.item(i).RegulationType;
							var regulationTypeText = "";
								if(regulationType == "B") { regulationTypeText =  "Basis regeling"; } else { regulationTypeText =  "Aanvullende regeling"; }
							var regulationDesc = results.rows.item(i).RegulationDesc;
		
							var aTag = "<div id=\"regulationcard"+i+"\"><a name=\"regulationAnchor"+i+"\" class=\"item item-icon-right\" onClick=\"fetchDataForItem('"+ i + "','"+ areaManagerId + "','" + regulationId + "');\">" +
								"(" + regulationTypeText + ") " + regulationDesc + " <i class=\"icon ion-chevron-right icon-accessory\"></i>" +
								"</a></div>";
							jQuery("#selectedItemRegulationsCard").append(aTag);			
						}
						jQuery("#selectedItemRegulationsCardLoading").remove();
					}
				}
				else {
					if(dbRequestKey == results.rows.item(0).dbKey) {
						var aTag = "<a class=\"item item-icon-right\" onClick=\"\">Geen regelingen gevonden.</a>";
						jQuery("#selectedItemRegulationsCard").append(aTag);			
						jQuery("#selectedItemRegulationsCardLoading").remove();
					}
				}
				showSelectedFeature();
			},function(tx, error) {
				alert('An Error Occured (Qry) : ' + error.message);
			}
		)});
	return 1;
}

function fetchDataForItem(cardId, areaManagerId, regulationId) {
	var query = "SELECT distinct * " + 
				"FROM parkingfare " + 
				"WHERE 1 "+
				" AND AreaManagerId='" + areaManagerId + "' AND RegulationId='"+ regulationId + "' " + 
				"	AND EndDateRegulation > '" + MAGIC_DATE + "' AND (EndDateFare > '" + MAGIC_DATE + "' OR EndDateFare=0 OR EndDateFare='') AND (EndDateFarePart > '" + MAGIC_DATE + "' OR EndDateFarePart=0 OR EndDateFarePart='')  " + 
				"	AND (EndDateTimeFrame > '" + MAGIC_DATE + "000000' OR EndDateTimeFrame='') AND (DateSpecialDay > '" + MAGIC_DATE + "' OR DateSpecialDay='') " + 
				"ORDER BY " + 
				"	RegulationType asc, " +
				"	RegulationId asc, " +
				"	DayTimeFrameNumber asc, " + 
				"	DateSpecialDay asc, " +
				"	substr('0000' || StartTimeTimeFrame,-4,300) asc, " + 	//fix omitted zeros at start of timestamp
				"	StartDateFarePart asc ";
	
	
	//close existing cards
	var openedSelectedCard = jQuery("#selectedRegulationDetails");
	if(openedSelectedCard.length > 0) {
		//simulate click on open item
		//openedSelectedCard.parent().children('a').click();
		var tmpVars = openedSelectedCard.parent().children('a').attr('onclick').replace("hideParkingFare(","").replace(");","").replace(/["']/g, "").split(',');
		hideParkingFare(tmpVars[0],tmpVars[1],tmpVars[2],true);
	}
	
	//open new (selected) card
	var selectedCard = jQuery("#regulationcard" + cardId);
	var selectedCardAnchor = jQuery("#regulationcard" + cardId + " a");
		selectedCardAnchor.css("background-color","#F6F6F6");
		selectedCardAnchor.attr('onclick', 'hideParkingFare('+cardId+',\''+areaManagerId+'\',\''+regulationId+'\');');
	var selectedCardAnchorArrow = jQuery("#regulationcard" + cardId + " a i")
		selectedCardAnchorArrow.removeClass('ion-chevron-right').addClass('ion-chevron-down');
		
	selectedCard.append("<div id=\"selectedRegulationDetails\" class=\"item\"></div>");
	var table = document.getElementById("selectedRegulationDetails");
	table.innerHTML = '';
	
	//fetch fare-rows
	db.transaction(function(tx){
		tx.executeSql(
			query, [], function(tx, results){
				var len = results.rows.length;
				var prevDay = '';
				var prevTimeFrame = '';
				
				for (var i=0; i<len; i++){
					var day = results.rows.item(i).DayTimeFrame;
					var startTimeTimeFrame = '0000' + results.rows.item(i).StartTimeTimeFrame;
						startTimeTimeFrame = startTimeTimeFrame.substr(startTimeTimeFrame.length - 4);
					var endTimeTimeFrame = '0000' + results.rows.item(i).EndTimeTimeFrame;
						endTimeTimeFrame = endTimeTimeFrame.substr(endTimeTimeFrame.length - 4);
					var startDurationFarePart = results.rows.item(i).StartDurationFarePart;
					var endDurationFarePart = results.rows.item(i).EndDurationFarePart;
					var amountFarePart = results.rows.item(i).AmountFarePart;
					var stepSizeFarePart = results.rows.item(i).StepSizeFarePart;
					var dateSpecialDay = results.rows.item(i).DateSpecialDay;
					
					if(dateSpecialDay !== "") {
						day = day + " (" + dateSpecialDay.substring(6,8) + "-" + dateSpecialDay.substring(4,6) + "-" + dateSpecialDay.substring(0,4) + ")";
					}

					if (day == prevDay) {
						//do nothing			
					} else {
						prevDay = day.toUpperCase();
						prevTimeFrame = '';
  						var aHeading = document.createElement('h3');
						aHeading.innerHTML = day.toUpperCase();
						table.appendChild(aHeading);
					}
					
					var aRow = document.createElement('div');
						aRow.setAttribute('class',"row");
					
					var timeFrame = "Tussen " + startTimeTimeFrame.substring(0,2) + ":" + startTimeTimeFrame.substring(2,4) + " en " + endTimeTimeFrame.substring(0,2) + ":" + endTimeTimeFrame.substring(2,4);
					if(timeFrame == prevTimeFrame) {
  						var aCol = document.createElement('div');
						aCol.setAttribute('class',"col col-5");
						aRow.appendChild(aCol);
  						aCol = document.createElement('div');
						aCol.setAttribute('class',"col");
						aCol.innerHTML = "Van " + startDurationFarePart + " t/m " + endDurationFarePart + " min.: € " + Number(amountFarePart).toFixed(2) + " per " + stepSizeFarePart + " min.";
						aRow.appendChild(aCol);						
					} else {
						prevTimeFrame = timeFrame;
  						var aCol = document.createElement('div');
						aCol.setAttribute('class',"col");
						aCol.innerHTML = timeFrame;
						aRow.appendChild(aCol);
						table.appendChild(aRow);			
						//add fare
						aRow = document.createElement('div');
							aRow.setAttribute('class',"row");
  						aCol = document.createElement('div');
						aCol.setAttribute('class',"col col-5");
						aRow.appendChild(aCol);
  						aCol = document.createElement('div');
						aCol.setAttribute('class',"col");
						aCol.innerHTML = "Van " + startDurationFarePart + " t/m " + endDurationFarePart + " min.: € " + Number(amountFarePart).toFixed(2) + " per " + stepSizeFarePart + " min.";
						aRow.appendChild(aCol);											
					}
					table.appendChild(aRow);
					scrollToRegulation("#regulationcard" + cardId);		
				}
			},function(tx, error) {
				alert('An Error Occured (Qry) : ' + error.message);
			}
		)});
		
	return 1;
}

function hideParkingFare(cardId, areaManagerId, regulationId, preventScroll) {
	//TODO
	var selectedCard = jQuery("#regulationcard" + cardId);
	var selectedCardAnchor = jQuery("#regulationcard" + cardId + " a");
		selectedCardAnchor.css("background-color","");
		selectedCardAnchor.attr('onclick', 'fetchDataForItem(\''+ cardId + '\',\''+ areaManagerId + '\',\'' + regulationId + '\');');
	var selectedCardAnchorArrow = jQuery("#regulationcard" + cardId + " a i")
		selectedCardAnchorArrow.removeClass('ion-chevron-down').addClass('ion-chevron-right');
	jQuery("#selectedRegulationDetails").remove();
	if(!preventScroll) {
		scrollToRegulation("#regulationcard" + cardId);
	}	
}

function onGeoLocationReceive(position) {
	if(typeof position.coords.latitude === 'undefined') { return false; }
	myLocation = new google.maps.LatLng(position.coords.latitude, position.coords.longitude);
	
	geolocationPoint.getGeometry().setCoordinates(ol.proj.transform([position.coords.longitude, position.coords.latitude], 'EPSG:4326', 'EPSG:3857'));
	geolocationPoint.setStyle(myLocationIconStyle);
	
	var factor = (view.getProjection().getPointResolution(view.getResolution(),view.getCenter())) / (view.getResolution());
	var radius = position.coords.accuracy / factor;
	geolocationAccuracy.getGeometry().setCenterAndRadius(ol.proj.transform([position.coords.longitude, position.coords.latitude], 'EPSG:4326', 'EPSG:3857'),radius);
	geolocationAccuracy.setStyle(myLocationStyle);

	if(!(document.getElementById('gmapMyLocation') == null)) { 
		if(mapControlsVisible()) {
			document.getElementById('gmapMyLocation').style.display='block';
		}
	}
}
function onGeoLocationReceiveError(error) {
	locationAvailable = false;
	if(!(document.getElementById('gmapMyLocation') == null)) { 
		if(!document.getElementById('gmapMyLocation')) { 
			document.getElementById('gmapMyLocation').style.display='none';
		}
	}
	//console.log('Geen GPS positie beschikbaar!');
}

function setBlocks() {
	blocks[1] = ol.extent.boundingExtent([[356850,7119955],[453150,6999045]]);
	blocks[2] = ol.extent.boundingExtent([[446850,7119955],[543150,6999045]]);
	blocks[3] = ol.extent.boundingExtent([[536850,7119955],[633150,6999045]]);
	blocks[4] = ol.extent.boundingExtent([[626850,7119955],[723150,6999045]]);
	blocks[5] = ol.extent.boundingExtent([[716850,7119955],[813150,6999045]]);
	blocks[6] = ol.extent.boundingExtent([[356850,7006955],[453150,6886045]]);
	blocks[7] = ol.extent.boundingExtent([[446850,7006955],[543150,6886045]]);
	blocks[8] = ol.extent.boundingExtent([[536850,7006955],[633150,6886045]]);
	blocks[9] = ol.extent.boundingExtent([[626850,7006955],[723150,6886045]]);
	blocks[10] = ol.extent.boundingExtent([[716850,7006955],[813150,6886045]]);
	blocks[11] = ol.extent.boundingExtent([[356850,6893955],[453150,6773045]]);
	blocks[12] = ol.extent.boundingExtent([[446850,6893955],[543150,6773045]]);
	blocks[13] = ol.extent.boundingExtent([[536850,6893955],[633150,6773045]]);
	blocks[14] = ol.extent.boundingExtent([[626850,6893955],[723150,6773045]]);
	blocks[15] = ol.extent.boundingExtent([[716850,6893955],[813150,6773045]]);
	blocks[16] = ol.extent.boundingExtent([[356850,6780955],[453150,6660045]]);
	blocks[17] = ol.extent.boundingExtent([[446850,6780955],[543150,6660045]]);
	blocks[18] = ol.extent.boundingExtent([[536850,6780955],[633150,6660045]]);
	blocks[19] = ol.extent.boundingExtent([[626850,6780955],[723150,6660045]]);
	blocks[20] = ol.extent.boundingExtent([[716850,6780955],[813150,6660045]]);
	blocks[21] = ol.extent.boundingExtent([[356850,6667955],[453150,6547045]]);
	blocks[22] = ol.extent.boundingExtent([[446850,6667955],[543150,6547045]]);
	blocks[23] = ol.extent.boundingExtent([[536850,6667955],[633150,6547045]]);
	blocks[24] = ol.extent.boundingExtent([[626850,6667955],[723150,6547045]]);
	blocks[25] = ol.extent.boundingExtent([[716850,6667955],[813150,6547045]]);
	
	for(i=1;i<(blocks.length);i++) {
		blocksLoaded[i]=false;
	}
}

function loadAdditionalBlocks() {
//console.log("loadAdditionalBlocks: " + (new Date().getTime()-debugTime));
	for(i=1;i<(blocks.length-1);i++) {
		if(ol.extent.containsCoordinate(blocks[i],view.getCenter())) {
			//correct block
			if(!(blocksLoaded[i])) {
				//show warning on screen
				loadingKMLWarning.style.display = 'block';
				
				//block was not loaded yet: load features and add to main layer
				var filename1 = 'kml/noPointsWA-';
				var filename2 = 'kml/onlyPointsWA-';
				if(i<10) {
					filename1 = filename1 + '0'; 
					filename2 = filename2 + '0'; 
				}
				filename1 = filename1 + i + '.kml';
				filename2 = filename2 + i + '.kml';
				var tmpSource1 = new ol.source.KML({
					  projection: 'EPSG:3857',
					  url: filename1
				});
				var tmpSource2 = new ol.source.KML({
					  projection: 'EPSG:3857',
					  url: filename2
				});
				//listen to change event on source, due to asynchronous loading of features
				listenerKeys[i+'-1'] = tmpSource1.on('change', function() { sourcePostProcess(tmpSource1, 'AREA',  (i+'-1')); }); 				
				listenerKeys[i+'-2'] = tmpSource2.on('change', function() { sourcePostProcess(tmpSource2, 'POINT', (i+'-2')); }); 				
				
				//features added, set loaded to true to prevent future loading
				blocksLoaded[i]=true;
			}
			return;
		}
	}
}
function sourcePostProcess(tmpSource, layerType, listenerKeyIndex) { 
	if (tmpSource.getState() == ol.source.State["READY"]) { 
		// the kml source is now loaded, we can unregister the "change" listener, 
		// and get the features from the source. 
		tmpSource.unByKey(listenerKeys[listenerKeyIndex]); 
		delete listenerKeys[listenerKeyIndex];
		
		switch(layerType) {
			case "POINT":
				tmpSource.getFeatures().forEach(function(aFeat) {
					applyVectorStyling(aFeat, false)
				});
				map.getLayers().getAt(2).getSource().addFeatures(tmpSource.getFeatures());
				break;
			case "AREA":
				tmpSource.getFeatures().forEach(function(aFeat) {
					aFeat.setStyle(defaultParkingArea);
				});
				map.getLayers().getAt(1).getSource().addFeatures(tmpSource.getFeatures());
				break;
			default:
				break;
		}
		//hide warning if everything is loaded
		if(Object.keys(listenerKeys).length < 1) { loadingKMLWarning.style.display = 'none'; }
		//simulate click on map if requested
		if(selectFeatureTrigger && (Object.keys(listenerKeys).length < 1)) { 
			selectFeatureTrigger = false;
			var lastClickedPosition = JSON.parse(storage.getItem("lastClickedPosition"));
			if(!(lastClickedPosition === null))
			{
				mapClickAtLocation(lastClickedPosition);
			}						
		}
	}
}

function findAddressByPosition(searchLatLng) {
	geocoder.geocode( { 'latLng': searchLatLng, 'language': 'nl' }, function(results, status) {
		if (status == google.maps.GeocoderStatus.OK) {
			showAddress(results[0].formatted_address);	
		} else {
			showAddress("");
		}
	});
}
function findPositionByAddress() {
	//ToDo: place marker and show progress indicator
	searchAddress.disabled=true;
	searchAddressButton.disabled=true;
	searchAddress.style.color = "#CCCCCC";
	searchAddressButton.style.color = "#CCCCCC";
	var address = document.getElementById('searchAddress').value;
	geocoder.geocode( { 'address': address}, function(results, status) {
		if (status == google.maps.GeocoderStatus.OK) {
			mapClickAtLocation(null, ol.proj.transform([results[0].geometry.location.lng(),results[0].geometry.location.lat()], 'EPSG:4326', 'EPSG:3857'));
			view.setCenter(ol.proj.transform([results[0].geometry.location.lng(),results[0].geometry.location.lat()], 'EPSG:4326', 'EPSG:3857'));
			view.setZoom(17);
		} else {
			if(status === "ZERO_RESULTS")
			{
				alert("Adres kon niet gevonden worden.");
			} else {
				alert("Unexpected error: " + status);
			}
		}
	});
	searchAddress.style.color = "";
	searchAddressButton.style.color = "";	
	searchAddress.disabled=false;
	searchAddressButton.disabled=false;
}

function showSelectedFeature() {
	changeCSSRule('#featureInfo','top', (geolocation.clientHeight - featureInfo.clientHeight + featureInfoSpacer.clientHeight)+'px');
	
//	featureInfo.setAttribute('style', baseStyle + 'top:'+(geolocation.clientHeight-featureInfo.clientHeight)+'px;');
	
/*
		selectedFeatureInfo.innerHTML = '' +
			'<div class="card" style="margin-bottom: 0px;">' +
	  			'<div class="item item-avatar">' + 
    				'<img src="img/logo.png">' + 
	    			'<h1 style="margin-top: 0px;margin-bottom: 0px;">' + feature.get('areaManagerIdDesc') + '</h1>' + 
    				'<span style="color: darkgray;">' + feature.get('areaIdDesc') + '</span>' + 
  				'</div>' + 
			'</div>';
*/
}

function regulationsVisible() {
	return (jQuery('#selectedItemRegulations').filter(':visible').length == 1); 
}
function showRegulations() {
	if(!(regulationsVisible())) {
		jQuery('#selectedItemRegulations').slideToggle({
		  direction: "up"
		}, 300);
		jQuery("#selectedItemContainer").removeClass().addClass("selectedItemContainerFull");
		jQuery("#regulationsScroll").height("80%");
		jQuery("#selectedItemContainer").height("inherit");
		//resize map
		mapCenter = map.getCoordinateFromPixel([gmap.clientWidth/2,gmap.clientHeight/2]);
		jQuery("#gmap").css("height", (gmap.clientHeight-selectedItemContainer.clientHeight));
		map.updateSize();
		view.setCenter(selectedMarker.getGeometry().getCoordinates());
	}
}
function hideRegulations() {
	if(regulationsVisible()) {
		jQuery('#selectedItemRegulations').slideToggle({
		  direction: "up"
		}, 300);
		jQuery("#selectedItemContainer").removeClass();
		jQuery("#regulationsScroll").height("0");
		jQuery("#selectedItemContainer").height("auto");
		//resize map
		jQuery("#gmap").css("height", '');
		map.updateSize();
		view.setCenter(mapCenter);
	}
}

function mapControlsVisible() {
	return (
				(jQuery('#searchAddressContainer').filter(':visible').length == 1)
//				&& (jQuery('#gmapMyLocation').filter(':visible').length == 1)
				&& (jQuery('.ol-zoom.ol-unselectable').filter(':visible').length == 1)
			);
}
function hideMapControls() {
	if(mapControlsVisible()) {
		jQuery(function () {
			jQuery('#searchAddressContainer').slideToggle({
			  direction: "down", duration: 300, queue: false
			});
			jQuery('#gmapMyLocation').slideToggle({
			  direction: "down", duration: 300, queue: false
			});
			jQuery('.ol-zoom.ol-unselectable').slideToggle({
			  direction: "down", duration: 300, queue: false
			});
		});		
	}
}
function showMapControls() {
	if(!(mapControlsVisible())) {
		jQuery(function () {
			jQuery('#searchAddressContainer').slideToggle({
			  direction: "down", duration: 300, queue: false
			});
			jQuery('#gmapMyLocation').slideToggle({
			  direction: "down", duration: 300, queue: false
			});
			jQuery('.ol-zoom.ol-unselectable').slideToggle({
			  direction: "down", duration: 300, queue: false
			});
		});		
	}
}

function selectedItemContainerVisible() {
	return (jQuery('#selectedItemContainer').filter(':visible').length == 1); 
}
function toggleSelectedItemContainer() {
	jQuery('#selectedItemContainer').slideToggle({
	  direction: "up"
	}, 300);
	jQuery("#selectedItemContainer").removeClass();
}
function hideSelectedItemContainer() {
	if(selectedItemContainerVisible()) {
		jQuery('#selectedItemContainer').slideToggle({
		  direction: "up"
		}, 300);
	}
	//also hide regulations
	if(regulationsVisible()) {
		hideRegulations();
	}
}




function setDefaultStyles() {
	invisibleStyle = new ol.style.Style({
		fill: new ol.style.Fill({
			color: 'rgba(0,0,0,0)'
		}),
		stroke: new ol.style.Stroke({
			color: 'rgba(0,0,0,0)',
			width: 0
		}),
		image: new ol.style.Icon({
			anchor: [0.5, 1],
			src: "img/transparent.png"
		})
	});

	defaultParkingArea = new ol.style.Style({
		fill: new ol.style.Fill({
			color: 'rgba(255, 0, 0, 0.2)'
		}),
		stroke: new ol.style.Stroke({
			color: 'rgba(255, 0, 0, 0.4)',
			width: 2
		})
	})	
	defaultParkingAreaSelected = new ol.style.Style({
		fill: new ol.style.Fill({
			color: 'rgba(0, 0, 255, 0.2)'
		}),
		stroke: new ol.style.Stroke({
			color: 'rgba(0, 0, 255, 0.4)',
			width: 2
		}),
		zIndex: 100
	})	

	defaultParkingIcon = new ol.style.Style({
		image: new ol.style.Icon({
			anchor: [0.5, 1],
			src: "img/parking.png"
		})
	});
	defaultParkingSelectedIcon = new ol.style.Style({
		image: new ol.style.Icon({
			anchor: [0.5, 1],
			src: "img/parking-active.png"
		})
	});

	defaultParkingGarageIcon = new ol.style.Style({
		image: new ol.style.Icon({
			anchor: [0.5, 1],
			src: "img/parkgarage.png"
		})
	});
	defaultParkingGarageSelectedIcon = new ol.style.Style({
		image: new ol.style.Icon({
			anchor: [0.5, 1],
			src: "img/parkgarage-active.png"
		})
	});

	defaultParkingTerrainIcon = new ol.style.Style({
		image: new ol.style.Icon({
			anchor: [0.5, 1],
			src: "img/parking.png"
		})
	});
	defaultParkingTerrainSelectedIcon = new ol.style.Style({
		image: new ol.style.Icon({
			anchor: [0.5, 1],
			src: "img/parking-active.png"
		})
	});

	defaultParkRideIcon = new ol.style.Style({
		image: new ol.style.Icon({
			anchor: [0.5, 1],
			src: "img/p_and_r.png"
		})
	});
	defaultParkRideSelectedIcon = new ol.style.Style({
		image: new ol.style.Icon({
			anchor: [0.5, 1],
			src: "img/p_and_r-active.png"
		})
	});
	
	myLocationStyle = new ol.style.Style({
		fill: new ol.style.Fill({
			color: 'rgba(200, 200, 200, 0.6)'
		}),
		stroke: new ol.style.Stroke({
			color: 'rgba(200, 200, 200, 0.2)',
			width: 2
		})		
	});
	myLocationIconStyle = new ol.style.Style({
		image: new ol.style.Icon({
			src: "img/current.png"
		})
	});

	selectedMarkerStyle = new ol.style.Style({
		image: new ol.style.Icon({
			anchor: [0.5, 1],
			src: "img/marker.png"
		})
	});
	
}


function resetApp() {
	document.location = "index.html";
}

function scrollToRegulation(elementId) {
	angular.element(jQuery("#regulationsScroll")).scope().scrollToRegulation(elementId);
}
